﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace graph
{
    public class graph
    {
        static void Main(string[] args)
        {
            SortedSet<Edge> priority = new SortedSet<Edge>();
            int numberOfNodes = 6;
            bool[] used = new bool[numberOfNodes + 1];
            List<Edge> mpdNodes = new List<Edge>();
            List<Edge> edges = new List<Edge>();

            var reader = new StreamReader("graph.in");
            int towns = 0;
            int roads = 0;
            int[,] afterReconstruction = null;

            using (reader)
            {
                string line = reader.ReadLine();
                int counter = 0;

                while (line != null)
                {
                    counter++;
                    string[] curLine = line.Split(' ');
                    if (counter == 1)
                    {
                        towns = int.Parse(curLine[0]);
                        roads = int.Parse(curLine[1]);
                        afterReconstruction = new int[towns, towns];
                    }
                    else if (counter <= roads + 1)
                    {
                        int first = int.Parse(curLine[0]);
                        int second = int.Parse(curLine[1]);
                        int dist = int.Parse(curLine[2]);
                        edges.Add(new Edge(first, second, dist, counter - 1));

                    }
                    else
                    {
                        for (int i = 0; i < towns; i++)
                        {
                            afterReconstruction[counter - roads - 2, i] = int.Parse(curLine[i]);
                        }
                    }
                    line = reader.ReadLine();
                }
            }

            for (int i = 0; i < edges.Count; i++)
            {
                if (edges[i].StartNode == edges[0].StartNode)
                {
                    priority.Add(edges[i]);
                }
            }
            used[edges[0].StartNode] = true;

            FindMinimumSpanningTree(used, priority, mpdNodes, edges);

            SaveTheTree(mpdNodes);
        }

        private static void SaveTheTree(List<Edge> usedEdges)
        {
            var writer = new StreamWriter("graph.out");
            using (writer)
            {
                foreach (var edge in usedEdges)
                {
                    writer.WriteLine(edge.Counter);
                }
            }
        }

        private static void FindMinimumSpanningTree(bool[] used, SortedSet<Edge> priority, List<Edge> mpdEdges, List<Edge> edges)
        {
            while (priority.Count > 0)
            {
                Edge edge = priority.Min;
                priority.Remove(edge);

                if (!used[edge.EndNode])
                {
                    used[edge.EndNode] = true;
                    mpdEdges.Add(edge);
                    AddEdges(edge, edges, mpdEdges, priority, used);
                }
            }
        }

        private static void AddEdges(Edge edge, List<Edge> edges, List<Edge> mpd, SortedSet<Edge> priority, bool[] used)
        {
            for (int i = 0; i < edges.Count; i++)
            {
                if (!mpd.Contains(edges[i]))
                {
                    if (edge.EndNode == edges[i].StartNode && !used[edges[i].EndNode])
                    {
                        priority.Add(edges[i]);
                    }
                }
            }
        }
    }

    class Edge : IComparable<Edge>
    {
        public int StartNode { get; set; }
        public int EndNode { get; set; }
        public int Weight { get; set; }
        public int Counter { get; set; }

        public Edge(int startNode, int endNode, int weight, int counter)
        {
            this.StartNode = startNode;
            this.EndNode = endNode;
            this.Weight = weight;
            this.Counter = counter;
        }

        public int CompareTo(Edge other)
        {
            int weightCompared = this.Weight.CompareTo(other.Weight);

            if (weightCompared == 0)
            {
                return this.StartNode.CompareTo(other.StartNode);
            }
            return weightCompared;
        }
    }
}
